@@ -70,7 +70,7 @@
 )]
 // END - Embark standard lints v0.4
 // crate-specific exceptions:
-#![deny(missing_crate_level_docs, missing_docs)]
+#![deny(missing_docs)]
 
 /// Bumped on protocol breakage.
 pub const PROTOCOL_VERSION: u16 = 1;
